#include "global.h"
using namespace std;
void GenerateFeatureSet(const char* dir,const int maxsize,const int label,int& start,Array2d<double>& features,Array2dC<int>& labels,const bool generatePCA,std::ofstream* pcaout=NULL)
{// for the UIUC scene classification set // Read images from one class (i.e. one directory)
    std::ostringstream buf;
    IntImage<double> im,temp;
    for(int i=0;i<maxsize;i++)
    {
        buf.str(""); buf<<dir; buf<<"image_"; buf.fill('0'); buf.width(4); buf<<i+1; buf<<".jpg";
        std::cout<<buf.str()<<std::endl;
        im.Load(buf.str());
        labels.buf[start]=label;
        for(int j=0;j<features.ncol;j++) features.p[start][j]=0;

        GenerateFeatureForSingleChannelImage(im,&features.p[start][0],(generatePCA && (i<100))?pcaout:NULL);
        // For UIUC scene classification data, use first 100 images of every class to generate PCA

        start++;
    }
}
void PrintMatrix(CvMat *Matrix,int Rows,int Cols , string filepath){
	FILE *output=fopen(filepath.c_str(),"w");
    for(int i=0;i<Rows;i++){
        for(int j=0;j<Cols;j++){
			fprintf(output,"%.7f ",cvGet2D(Matrix,i,j).val[0]);
        }
    }
	fclose(output);
}
void Scenes()
{
    bool generatePCA = false; // whether we collect training data for PCA

    lf.Load("lf_UIUC.txt",40,256); NormalizeLoadFactors(lf);

    sizeCV = 5;
    extra = 2;
    splitlevel = 2;
    splitratio[0] = 1.0; splitratio[1] = 1.0; splitratio[2] = 1.35;

    const int nClass = 8;
	const int imagesizes[nClass] = {360,328,260,308,374,410,292,356};
    const char* imagenames[nClass] = {"../Data/scene/MITcoast/",
                           "../Data/scene/MITforest/","../Data/scene/MIThighway/","../Data/scene/MITinsidecity/",
                           "../Data/scene/MITmountain/","../Data/scene/MITopencountry/","../Data/scene/MITstreet/",
                           "../Data/scene/MITtallbuilding/"}; 
    const int classLabels[nClass] = { 0, 1, 2, 3, 4, 5, 6, 7 };
    const int totalNum = 2688;

    Array2d<double> features;Array2dC<int> labels;
    features.Create(totalNum,splitlevel?(lf.nrow+extra)*splitsize[splitlevel]:(256+extra));
    labels.Create(1,features.nrow);

    std::ofstream pcaout;
    if(generatePCA) pcaout.open("pcatrain.txt"); // use first 100 images in each class to generate PCA
    int start=0;
    for(int i=0;i<nClass;i++) 
        GenerateFeatureSet(imagenames[i],imagesizes[i],classLabels[i],start,features,labels,generatePCA,&pcaout);
    assert(start==features.nrow);
	if(generatePCA) 
    {
        pcaout.close();
		int vn=0;
		float temp=0;
		FILE *file=fopen("pcatrain.txt","r");
		while(!feof(file)){
			for(int j=0 ; j<256 ; j++){
				fscanf(file,"%f ",&temp);
			}
			vn++;
		}
		fclose(file);
		float *Coordinates=new float[vn*256] ;
		for(int i=0;i<vn*256;i++){
			Coordinates[i]=0;
		}
		printf("vn*256=%d\n",vn*256);
		int i=0;
		file=fopen("pcatrain.txt","r");
		while(!feof(file)){
			for(int j=0 ; j<256 ; j++){
				fscanf(file,"%f ",&Coordinates[i]);
				i++;
			}
		}
		fclose(file);
	    CvMat *Vector1=cvCreateMat(vn,256,CV_32FC1);
		CvMat *AvgVector=cvCreateMat(1,256,CV_32FC1);
		CvMat *EigenValue_Row=cvCreateMat(256,1,CV_32FC1);
		CvMat *EigenVector=cvCreateMat(256,256,CV_32FC1);
		cvSetData(Vector1,Coordinates,Vector1->step);
	    cvCalcPCA(Vector1,AvgVector,EigenValue_Row,EigenVector,CV_PCA_DATA_AS_ROW);
		PrintMatrix(EigenVector,40,256,"lf_UIUC.txt");
		delete [] Coordinates;
		return ;
    }
    
    const int size=features.nrow;
    Array2dC<bool> train(1,size);
    Array2dC<double> minmax(2,features.ncol);
    Array2dC<double> scaleratio(1,features.ncol); PrepareScaleRatio(scaleratio);

    unsigned seed = 1194575810; // unsigned seed=(unsigned)time(NULL);  // use 1194575810 to reproduce results in paper
    std::cout<<seed<<std::endl;
    my_srand(seed);
    for(int round=0;round<sizeCV;round++)
    {
        for(int i=0;i<size;i++) train.buf[i]=false; // train/test set division

        start=0;
        for(int i=0;i<nClass;i++)
        {
            int end = start;
            while(end<size && labels.buf[end]==labels.buf[start]) end++;

            Array2dC<int> perm(1,end-start);
            GenerateRandomPermutation(perm.buf,end-start);
            for(int j=0;j<100;j++) train.buf[start+perm.buf[j]] = true;
            start = end;
        }
        FindMinMaxValue(features,train,minmax);
		cout<<"Writing the round "<<round+1<<" testing and training files."<<endl;
        std::ostringstream buf;
        buf.str(""); buf<<"train"; buf<<round+1; buf<<".txt";
        std::ofstream out(buf.str().c_str());
        for(int i=0;i<features.nrow;i++)
        {
            if(train.buf[i]==false) continue;
            out<<labels.buf[i]<<" ";
            for(int j=0;j<features.ncol;j++) 
                out<<j+1<<":"<<((features.p[i][j]-minmax.p[0][j])*minmax.p[1][j]-1.0)*scaleratio.buf[j]<<" ";
            out<<std::endl;
        }
        out.close();
		
        buf.str(""); buf<<"test"; buf<<round+1; buf<<".txt";
        std::ofstream out2(buf.str().c_str());
        for(int i=0;i<features.nrow;i++)
        {
            if(train.buf[i]==true) continue;
            out2<<labels.buf[i]<<" ";

            for(int j=0;j<features.ncol;j++) 
                out2<<j+1<<":"<<((features.p[i][j]-minmax.p[0][j])*minmax.p[1][j]-1.0)*scaleratio.buf[j]<<" ";
            out2<<std::endl;
        }
        out2.close();
    }
}
